/*
  ServerCommunicationProtocol is a class which is a layer between
  raw data stream flow and structured messages.

  Alexander Podoplelov, RISC-Linz, 1998.
*/

// ****************************************************
import java.io.*;
import java.util.*;
import java.net.*;
// ****************************************************

abstract public class CommunicationProtocol {
    protected   BufferedReader     in;
    protected   BufferedWriter     out;

    
    public CommunicationProtocol (BufferedReader in, BufferedWriter out){
	this.in = in;
	this.out = out;
    }

    // Getting the message according to protocol
    abstract public Message GetMessage ()
	  throws SessionDoneException, UnknownKeyException, IOException;

    public void SendMessage (int key, String what) {
	try {
	    String skey = Message.Map (key);
	    out.write (skey + "\n");
	    out.write (what + "\n");
	    out.flush();
	} 
	catch (UnknownKeyException e) {}
	catch (IOException e1) {}
    }


    public void SendMessage (int key, int what) {
	try {
	    String skey  = Message.Map (key);
	    String swhat = Message.Map (what);
	    out.write (skey  + "\n");
	    out.write (swhat + "\n");
	    out.flush();
	} 
	catch (UnknownKeyException e) {}
	catch (IOException e1) {}
    }

    public void SendMessage (int key, Vector what) {
	try {
	    String skey = Message.Map (key);
	    int    N;
	    out.write (skey + "\n");
	    if (what == null)
		N = 0;
	    else
		N = what.size();
	    out.write (Integer.toString(N) + "\n");
	    if (N > 0)
		for (int i = 0; i < what.size(); i++) {
		    Object tosend = (Object)what.elementAt (i);
		    out.write (tosend.toString() + "\n");
		}
	    out.flush();
	} 
	catch (UnknownKeyException e) {}
	catch (IOException e1) {}
    }
}

// ****************************************************

