/*
  Configuration class contains program properties. It handels their reading/
  writing.

  Alexander Podoplelov, RISC-Linz, 1998.
*/

// ****************************************************
import java.io.*;
import java.net.*;
import java.text.*;
import java.sql.*;
import java.util.*;
// ****************************************************

public class Configuration {

    public   static String         FILE_NAME    = "Generator.conf";
    public   static String         TEST_URL_KEY = "DO NOT REMOVE THIS FILE!";

    private  static DateFormat     date_format;
    private  static DateFormat     short_date_format;
    private  static Calendar       calendar;
    private  static Properties     configuration;

    public   static String         dbURL;
    public   static String         WWW_base;
    public   static String         HardDrive_base;
    public   static String         HD_contr_parts;
    public   static String         IndexTemplate;
    public   static String         DataTemplate;
    public   static String         BannerTemplate;
    public   static String         NoPartsHTML;
    public   static String         ErrorHTML;
    public   static String         DefDataFile;

    // Initialization for static variables
    static {
	try {
	    SecurityManager sm = System.getSecurityManager();
	    if (sm != null)
		sm.checkRead(FILE_NAME);
	    date_format = DateFormat.getDateInstance(DateFormat.LONG);
	    short_date_format = DateFormat.getDateInstance(DateFormat.SHORT);
	    calendar = new GregorianCalendar();
	    configuration  = new Properties();
	    FileInputStream f = new FileInputStream(FILE_NAME);
	    configuration.load (f);
	    f.close();
	    // configuration.list (System.out);
	} catch (IOException e) {
	    abort ("Generator: Cannot load configuration \"" +
		   FILE_NAME +"\"");
	} catch (SecurityException e1) {
	    abort ("SecurityManager:" + e1.toString());
	}

	dbURL          = getValueOf ("database.url");
	WWW_base       = getValueOf ("www.basedoc");
	HardDrive_base = getValueOf ("meetings.dir");
	HD_contr_parts = getValueOf ("raw_data");
	IndexTemplate  = getValueOf ("template.index");;
	DataTemplate   = getValueOf ("template.data");
	BannerTemplate = getValueOf ("template.banner");
	NoPartsHTML    = getValueOf ("error.no_parts");
	ErrorHTML      = getValueOf ("error.no_contributions");
	DefDataFile    = getValueOf ("default.banner");
    }

    public static void Check() {
	System.out.println ("Checking configuration.");

	if (dbURL == null)
	    abort ("SQL database URL \"database.url\" is not specified");
	else {
	    try {
		Connection c = MeetingSystemDB.getConnection();
		try { Thread.sleep (1000); } catch (InterruptedException e) {}
		c.close();
	    } catch (SQLException e) {
		abort (e.toString());
	    }
	    System.out.println ("\"database.url\" \t\t\tis Ok.");
	}

	if (WWW_base == null)
	    abort ("WWW prefix \"www.basedoc\" is not specified");
	else {
	    String remote_doc_test = getValueOf ("remote.testdoc");
	    if (remote_doc_test == null)
		abort ("\"remote.testdoc\" variabe not found in the config.");
	    else {
		URL test_url = null;
		try {
		    test_url = new URL (WWW_base + remote_doc_test);
		} catch (MalformedURLException e) {
		    abort (e.toString());
		}
		BufferedReader f = null;
		try {
		    f = new BufferedReader
			(new InputStreamReader (test_url.openStream()));
		} catch (IOException e) {
		    abort ("cannot read www " + test_url.toString() + "\n" +
			   e.toString());
		}
		try {
		    String s = f.readLine();
		    if (s == null)
			abort ("File in " + test_url.toString() +
			       "contains no data");
		    if (!s.equals(TEST_URL_KEY))
			abort ("URL key \"" + s + "\" does not much \"" +
			       TEST_URL_KEY + "\"");
		} catch (IOException e) {
		    abort ("cannot read www " + test_url.toString() + "\n" +
			   e.toString());
		}
		try { f.close(); } catch (IOException e) {}
	    } // remote_doc_test
	    System.out.println ("\"www.basedoc\" \t\t\tis Ok.");
	}

	if (HardDrive_base == null)
	    abort ("Meetings directory \"meetings.dir\" is not specified");
	else {
	    try {
		File f = new File (HardDrive_base);
		if (!f.exists())
		    abort ("\"" + HardDrive_base + "\" does not exist");
	    } catch (SecurityException e) {
		abort (e.toString());
	    }
	    System.out.println ("\"meetings.dir\" \t\t\tis Ok.");
	}

	if (HD_contr_parts == null)
	    abort ("Meetings directory \"raw_data\" is not specified");
	else {
	    try {
		File f = new File (HardDrive_base, HD_contr_parts);
		if (!f.exists())
		    abort ("\"" + f.getPath() + "\" does not exist");
	    } catch (SecurityException e) {
		abort (e.toString());
	    }
	    System.out.println ("\"raw_data\" \t\t\tis Ok.");
	}

	if (IndexTemplate == null)
	    abort ("WWW prefix \"template.index\" is not specified");
	else {
	    try {
		File f = new File (IndexTemplate);
		if (!f.exists())
		    abort ("\"" + IndexTemplate + "\" does not exist");
	    } catch (SecurityException e) {
		abort (e.toString());
	    }
	    System.out.println ("\"template.index\" \t\tis Ok.");
	}
	if (DataTemplate == null)
	    abort ("Template \"template.data\" is not specified");
	else {
	    try {
		File f = new File (DataTemplate);
		if (!f.exists())
		    abort ("\"" + DataTemplate + "\" does not exist");
	    } catch (SecurityException e) {
		abort (e.toString());
	    }
	    System.out.println ("\"template.data\" \t\tis Ok.");
	}
	if (BannerTemplate == null)
	    abort ("Template \"template.banner\" is not specified");
	else {
	    try {
		File f = new File (BannerTemplate);
		if (!f.exists())
		    abort ("\"" + BannerTemplate + "\" does not exist");
	    } catch (SecurityException e) {
		abort (e.toString());
	    }
	    System.out.println ("\"template.banner\" \t\tis Ok.");
	}
	if (NoPartsHTML == null)
	    abort ("Error Doc. \"error.no_parts\" is not specified");
	else {
	    File f = new File (HardDrive_base, NoPartsHTML);
	    if (!f.exists())
		abort ("File \"" + f.getPath() + " does not exist");
	    System.out.println ("\"error.no_parts\" \t\tis Ok.");
	}
	if (ErrorHTML == null)
	    abort ("Error Doc. \"error.no_contributions\" is not specified");
	else {
	    File f = new File (HardDrive_base, ErrorHTML);
	    if (!f.exists())
		abort ("File \"" + f.getPath() + " does not exist");
	    System.out.println ("\"error.no_contributions\" \tis Ok.");
	}

	configuration  = null;
    }

    public static String Format (java.util.Date d) {
	return date_format.format (d);
    }
    
    public static String ShortFormat (java.util.Date d) {
	return d.toString();
// 	calendar.setTime (d);
// 	return Integer.toString(calendar.get (Calendar.HOUR)) + ":" +
// 	    Integer.toString(calendar.get (Calendar.MINUTE));
// 	return short_date_format.format (d);
    }
    
    public static String getValueOf (String key) {
	return configuration.getProperty (key);
    }

    public static void abort (String s) {
	System.err.println (s + "\nAborting.");
	System.exit (-1);
    }
}

class QuickSortAlg {
    // The Quick Algorithm Class

    void QuickSort (int quickarray[], int lobound,int hibound) {
        int lo = lobound;
        int hi = hibound;
        int mid;    //midpoint

        if (hibound > lobound) {

	    /* Arbitrarily establishing partition element as the midpoint of
	     * the array.
	     */
            mid = quickarray [(lobound + hibound) / 2];

	    // loop through the array until indices cross
            while (lo <= hi) {

                /* find the first element that is greater than or equal to
		 * the partition element starting from the left Index.
		 */
                while (lo < hibound && quickarray[lo] < mid) ++lo;

                /* find an element that is smaller than or equal to
		 * the partition element starting from the right Index.
		 */
                while (hi > lobound && quickarray[hi] > mid) --hi;

                // if the indexes have not crossed, swap

                if (lo <= hi) {
                    int tempno = quickarray [lo];  // swap y[lo] and y[hi]
                    quickarray [lo] = quickarray [hi];
                    quickarray [hi] = tempno;
                    ++lo;
                    --hi;
                }
            }

	    /* If the right index has not reached the left side of array
	     * must now sort the left partition.
	     */
	    if (lobound < hi)
		QuickSort (quickarray, lobound, hi);

	    /* If the left index has not reached the right side of array
	     * must now sort the right partition.
	     */
	    if (lo < hibound)
		QuickSort (quickarray, lo, hibound);

	}
    }
}


// ****************************************************

