/*
  ContributionItem data type is for MeetingProtocol data structures. It
  corresponds to the data which reside in SQL data server.

  Alexander Podoplelov, RISC-Linz, 1998.
*/

// ****************************************************
import java.io.*;
import java.sql.*;
import java.lang.*;
import java.util.*;
// ****************************************************

public class  ContributionItem extends Object {

    private  int    id;
    private  int    meeting_id;
    private  String user_name;
    private  int    parent;
    private  String contr_name;
    private  Timestamp time_stamp;
    private  String start_time;
    private  String annotation;
    private  Vector contr_parts; // Vector of ContributionPartItems
    private  Vector references;  // Vector of ContributionItems
    private  Vector childs;      // Vector of ContributionItems
    private  String watermark;   // Watermark for other references in HTML doc.
    
    public ContributionItem (int id, int m_id, String u_name,
			     int parent, String contr_name,
			     Timestamp time_stamp, String start_time,
			     String annotation) {
	this.id           = id;
	this.meeting_id   = m_id;
	this.user_name    = u_name;
	this.parent       = parent;
	this.contr_name   = contr_name;
	this.time_stamp   = time_stamp;
	this.start_time   = start_time;
	this.annotation   = annotation;
	this.references   = null;
	this.childs       = null;
	this.contr_parts  = null;
	this.watermark    = null;
    }

    public void Write (BufferedWriter f, String field) throws IOException {
	if (field == null) return;
	if (field.equals("user_name")) f.write (user_name);
	else if (field.equals("contribution_name"))
	    f.write (contr_name);
	else if (field.equals("time_stamp"))
	    f.write (time_stamp.toString());
	else if (field.equals("start_time"))
	    f.write (start_time.substring (11, 16));
	else if (field.equals("annotation"))
	    f.write (annotation);
    }

    public void WriteReferences (BufferedWriter f) throws IOException {
	if (references != null)
	    for (int i = 0; i < references.size(); i++) {
		Reference r = (Reference)references.elementAt (i);
		if (i == 0)
		    f.write ("References:<BR>\n");
		f.write ("<A HREF=\"" + r.to_ci.getWaterMark(false) + "\">");
		r.to_ci.Write (f, "contribution_name");
		f.write ("</A><BR>\n");
	    }
    }

    public int getID() {
	return id;
    }

    public void setContributionParts (Vector cp) {
	contr_parts = cp;
    }

    public Vector ContributionParts () {
	return contr_parts;
    }

    public Vector References () {
	return references;
    }

    public Vector Childs () {
	return childs;
    }

    public void setReferences (Vector r) {
	references = r;
    }

    public void NewChilds () {
	childs = new Vector();
    }

    public int ID () {
	return id;
    }

    public int Parent () {
	return parent;
    }

    public String getWaterMark(boolean shortcat) {
	if (shortcat) {
	    int separator = watermark.indexOf ((int)'#');
	    if (separator < 0)
		return watermark;
	    return watermark.substring (separator + 1);
	} else
	    return watermark;
    }

    public void setWaterMark (String wm) {
	watermark = wm;
    }
}
// ****************************************************
