/*
  DirectoryWatchDog keeps generated directory clean. It deletes
  those directories for which there was not access for a certain time.

  Alexander Podoplelov, RISC-Linz, 1998.
*/

// ****************************************************
import java.io.*;
import java.util.*;
// ****************************************************

public class DirectoryWatchDog extends Thread {
    public  static final int DELAY         = 30 * 1000; // 30 sec.
    //    public  static final int DELAY         = 10 * 60 * 1000; // 10 min
    public  static final int EXPIRATION    = 1 * 24 * 60 * 60 * 1000;// 1 day.
    //    public  static final int EXPIRATION    = 7 * 24 * 60 * 60 * 1000; // 1 week

    private File             root_dir;

    public DirectoryWatchDog (File root_dir) {
	this.super();
	this.root_dir   = root_dir;
	this.start();
    }

    public void run() {
	MeetingProtocolGenerator.Log
	    ("Thread \"eater\" for old meeting directories has been started");
	while (true) {
	    try { Thread.sleep (DELAY); } catch (InterruptedException e) {}
	    Revise();
	}
    }

    private void Revise() {
	String[] name = root_dir.list();
	for (int i = 0; i < name.length; i++) {
	    File f = new File (root_dir, name[i]);
	    if (f.isDirectory())
		Checkup (f);
	}
	name = null; // for garbage collector
    }

    private void Checkup (File f) {
	String name        = f.getName();
	String access_time = null;
	synchronized (Generator.MeetingHash) {
	    access_time = Generator.MeetingHash.getProperty (name);
	}
	if (access_time == null) return;
	long access = 0;
	try {
	    access = Long.parseLong (access_time);
	} catch (NumberFormatException e) {
	    return;
	}
	long difference = new Date().getTime() - access;
	if (difference > EXPIRATION) {
	    MeetingProtocolGenerator.Log ("Difference: " +
					  difference / 1000.0 +
					  " sec. Deleted \"" +
					  f.getName() + "\"");
	    Delete (f);
	    synchronized (Generator.MeetingHash) {
		Generator.MeetingHash.remove (name);
		Generator.SaveMeetingHash();
	    }
	}
    }

    private void Delete (File f) {
	String[] name = f.list();
	for (int i = 0; i < name.length; i++) {
	    File f1 = new File (f, name[i]);
	    if (f1.isDirectory())
		Delete (f1);
	    else
		f1.delete();
	}
	if (MeetingProtocolGenerator.securitymanager != null)
	    try {
		MeetingProtocolGenerator.securitymanager.checkDelete
		    (f.getPath());
	    } catch (SecurityException e) {
		MeetingProtocolGenerator.Log (e.toString());
	    }
	if (!f.delete())
	    MeetingProtocolGenerator.Log
		("*** Cannot delete \"" + f.getName() + "\"");
	//	MeetingProtocolGenerator.Log ("Deleted \"" + f.getName() + "\"");
	name = null; // for garbage collector
    }
}
// ****************************************************
