/*
  DocumentProtocol class parses template files and generates high
  order messages.

  Alexander Podoplelov, RISC-Linz, 1998.
*/

// ****************************************************
import java.io.*;
import java.util.*;
// ****************************************************

public class DocumentProtocol {
    private StreamTokenizer    st      = null;
    private StringTokenizer    str_tok = null;

    public DocumentProtocol (Reader r) {
	st = new StreamTokenizer (r);
	st.resetSyntax();
	st.wordChars('\u0000', '\uFFFF');
	st.whitespaceChars('$', '$');
	st.lowerCaseMode (false);
	st.eolIsSignificant (false);
    }

    private String GetToken () throws IOException, SessionDoneException {
	if (str_tok == null || str_tok != null && !str_tok.hasMoreTokens()) {
	    if (st.nextToken() == StreamTokenizer.TT_EOF) {
		st      = null;
		str_tok = null;
		throw new SessionDoneException ();
	    } else
		str_tok = new StringTokenizer (st.sval, "[]");
	}
	return str_tok.nextToken();
    }

    public DocMessage GetMessage ()
	throws DocProtException, SessionDoneException
    {
	String   t     = null;
	try {
	    t = GetToken();
	} catch (IOException e) {
	    throw new DocProtException ("Document Protocol: " + e.toString());
	}
	try {
	    if (t.equals("COMMAND")) {
		t = GetToken();
		if (t.equals("HEADER"))
		    return new DocMessage (Message.CMD_HEADER);
		else if (t.equals("FOOTER"))
		    return new DocMessage (Message.CMD_FOOTER);
		else if (t.equals("END"))
		    return new DocMessage (Message.CMD_END);
		else if (t.equals("ITERATE"))
		    return new DocMessage (Message.CMD_ITERATE);
		else if (t.equals("LIST_START"))
		    return new DocMessage (Message.CMD_LIST_START);
		else if (t.equals("LIST_END"))
		    return new DocMessage (Message.CMD_LIST_END);
		else if (t.equals("LIST_ITEM"))
		    return new DocMessage (Message.CMD_LIST_ITEM);
		else if (t.equals("REFERENCES"))
		    return new DocMessage (Message.CMD_REFERENCES);
		else if (t.equals("GENERATED_TIME"))
		    return new DocMessage (Message.TIMESTAMP);
		else
		    throw new DocProtException ("Unknown COMMAND " + t);
	    }
	    else if (t.equals("FIELD"))
		return new DocMessage (Message.CMD_FIELD);
	    else if (t.equals("WATERMARK"))
		return new DocMessage (Message.WATERMARK);
	    else
		return new DocMessage (Message.DATA, t);
	} catch (IOException e1) {
	    throw new DocProtException (e1.toString());
	} catch (SessionDoneException e2) {
	    throw new DocProtException ("Unexpected EOF in Document Protocol");
	}
    }
}
// ****************************************************
