/*
  MeetingSystemDB is a class which does a general job
  concerning the concrete database. It registers the Openlink
  driver and prepares a vector of shelled connections.
  The main purpose of this class is to expose the less overloaded
  connection.

  Alexander Podoplelov, RISC-Linz, 1998.
*/

// ****************************************************
import java.io.*;
import java.sql.*;
import java.util.*;
// ****************************************************



public final class MeetingSystemDB {

    //    private static final int             TOTAL_CONNECTIONS = 3;

    public  static       ShelledConnection connection       = null;

    // Simulation data
    public  static       Vector          Meetings           = null;
    public  static       Vector          Tools              = null;
    public  static       Vector          Users              = null;
    public  static       Vector          Contributions      = null;
    public  static       Vector          ContributionParts  = null;



    // at Class initialisation register the JDBC driver
    static {
	try {
	    // Add the OpenLink JDBC driver to the system properties
            Properties p = System.getProperties ();
            p.put ("jdbc.drivers", "openlink.jdbc.Driver");
            System.setProperties (p);
	}
	catch (Exception e) {
	    System.err.println("ERROR: Cannot register openlink.jdbc.Driver");
	    System.exit (0);
	}
	
	// Setting up the serviced database connection
	connection = new ShelledConnection ();
	
	// Allocate some data for debugging purposes
	MakeSimulationData();
    }

    public static synchronized Connection getConnection() throws SQLException {
	return DriverManager.getConnection (Configuration.dbURL);
    }

    public static ShelledConnection RequestConnection() {
	return connection;
    }

    public static int getTableCode (String Table) {
	if (Table.equals("meeting"))                 return 1;
	else if (Table.equals("meeting_user"))       return 2;
	else if (Table.equals("tool"))               return 3;
	else if (Table.equals("contribution"))       return 4;
	else if (Table.equals("contribution_part"))  return 5;
	return 0;
    }

    public static String getTableName (int Table) {
	switch (Table) {
	case 1: return "meeting";
	case 2: return "meeting_user";
	case 3: return "tool";
	case 4: return "contribution";
	case 5: return "contribution_part";
	}
	return null;
    }
    
    private static void MakeSimulationData() { /*
	Meetings          = new Vector();
	Tools             = new Vector();
	Users             = new Vector();
	Contributions     = new Vector();
	ContributionParts = new Vector();

	Meetings.addElement
	    (new MeetingItem(1,
			     "Computation in Algebra and Geometry",
			     "Prague, Check Republic.",
    java.sql.Date.valueOf("1998-08-09"), // java.sql.Date("Aug 9 1998"),
    java.sql.Date.valueOf("1998-08-11"), // java.sql.Date("Aug 11 1998"),
"Computer algebra gives effective possibility to realize algorithms \n" +
"for symbolic calculation in different branches of mathematics.\n" +
"This session aims to join researches in the following directions\n" +
"connected with computation in Algebra and Geometry:\n" +
"- Variations on Groebner bases, normal forms and algebraic \n" +
"  representations;\n" +
"- Complexity of Groebner bases constructions and algebraic\n" +
"  computation;\n" +
"- Computing in Ring Theory (associative rings, Lie algebras \n" +
"  and superalgebras,\n" +
"  quantum algebras, Jordan and other non-associative rings);\n" +
"- Computing with groups;\n" +
"- A computational approach to Semigroups, Automata and Formal \n" +
"  Languages;\n" +
"- Computational Algebraic Geometry and Commutative Algebra;\n" +
"- Computing with Lattices, Projective, Incidence and Finite \n" +
"  Geometries;\n" +
"- Computing Graph Theory;\n" +
"- Standard bases in  Differential and Difference Algebra;\n" +
"- Finite fields and rings, Quasigroups, Coding Theory."));
	Meetings.addElement (new MeetingItem(2,
"Performance Models for Discrete Event Systems with Synchronizations: " +
"Formalisms and Analysis Techniques.",
"Jaca, Spain",
java.sql.Date.valueOf("1998-09-03"), // java.sql.Date("Sep 3 1998"),
java.sql.Date.valueOf("1998-09-11"), // java.sql.Date("Sep 11 1998"),
"MATCH (Modelling and Analysis of Time Constrained and\n" +
"Hierarchical Systems) is a Human Capital and Mobility\n" +
"initiative, sponsored by the European Union. One of its\n" +
"objectives is the organisation of two open\n" +
"complementary Advanced Schools concerning Discrete\n" +
"Event Dynamic Systems (DEDS), focusing one on\n" +
"performance modelling and evaluation, and the other on\n" +
"modelling and verification. Each school can be attended\n" +
"independently.\n"));

	Tools.addElement (new ToolItem (1, "excel"));
	Tools.addElement (new ToolItem (2, "winword"));
	Tools.addElement (new ToolItem (3, "canon"));

	Users.addElement (new UserItem("J. Smith", "Red56dOwn", "Prof. in Mathematics"));
	Users.addElement (new UserItem("Y. Dallery", "hj6s4a", "Prof. in CS"));
	Users.addElement (new UserItem("E. Teruel", "k7sa6ga", "Prof. in CS"));
	Users.addElement (new UserItem("M. Silva", "Stratostat85", "Prof. in Physics"));

	Vector annotation = new Vector();
	Contributions.addElement (new ContributionItem(1, 1, "J. Smith", 1, 0, "Contribution 1", java.sql.Timestamp.valueOf("1998-09-03 15:53:33.00"), annotation));
	Contributions.addElement (new ContributionItem(2, 1, "Y. Dallery", 2, 0, "Contribution 2", java.sql.Timestamp.valueOf("1998-09-03 15:53:33.00"),annotation));
	Contributions.addElement (new ContributionItem(3, 2, "E. Teruel", 3, 0, "Contribution 3", java.sql.Timestamp.valueOf("1998-09-03 15:53:33.00"), annotation));
	Contributions.addElement (new ContributionItem(4, 2, "M. Silva", 1, 0, "Contribution 4", java.sql.Timestamp.valueOf("1998-09-03 15:53:33.00"), annotation));

	ContributionParts.addElement (new ContributionPartItem(1, 1, "text/plain", "", "file:/home/ppodople/Mail/Friends/Mircea/20"));
	ContributionParts.addElement (new ContributionPartItem(2, 2, "text/plain", "", "file:/home/ppodople/Mail/Friends/Mircea/21"));
	ContributionParts.addElement (new ContributionPartItem(3, 3, "text/plain", "", "file:/home/ppodople/Mail/Friends/Mircea/22"));
	ContributionParts.addElement (new ContributionPartItem(4, 4, "text/plain", "", "file:/home/ppodople/Mail/Friends/Mircea/23"));
					       */
    }
}




