/*
  Message class provides implementation for structures of messages.

  Alexander Podoplelov, RISC-Linz, 1998.
*/

// ****************************************************
import java.applet.*;
import java.awt.*;
import java.util.*;
import java.net.*;
// ****************************************************

abstract class Message {
    // Input keywords
    public static final int HANDSHAKE        =  1;
    public static final int MEETINGLIST      =  2;
    public static final int GENERATE         =  3;
    public static final int TOOLS            =  4;
    public static final int CLOSE            =  5;

    // Output keywords
    public static final int CONFIRMATION     =  14;
    public static final int ERROR            =  15;
    public static final int DOCURL           =  16;

    // String Tokenizer keywords
    public static final int CMD_HEADER       =  1;
    public static final int CMD_FOOTER       =  2;
    public static final int CMD_FIELD        =  3;
    public static final int CMD_END          =  4;
    public static final int CMD_ITERATE      =  5;
    public static final int CMD_LIST_START   =  6;
    public static final int CMD_LIST_END     =  7;
    public static final int CMD_LIST_ITEM    =  8;
    public static final int CMD_REFERENCES   =  9;
    public static final int WATERMARK        =  10;
    public static final int DATA             =  11;
    public static final int TIMESTAMP        =  12;
 

    protected   int     key;
    protected   Vector  data;

    public Message (int key) {
	this.key = key;
	data = new Vector();
    }

    public Message (int key, String arg) {
	this.key = key;
	data = new Vector();
	data.addElement (arg);
    }

    public Message (String key) throws UnknownKeyException {
	this.key = Map(key);
	data = new Vector();
    }

    public static int Map (String key) throws UnknownKeyException {
	if (key.equals("HANDSHAKE"))     return HANDSHAKE; else
	if (key.equals("MEETINGLIST"))   return MEETINGLIST; else
	if (key.equals("GENERATE"))      return GENERATE; else
	if (key.equals("TOOLS"))         return TOOLS; else
	if (key.equals("CLOSE"))         return CLOSE; else
	if (key.equals("CONFIRMATION"))  return CONFIRMATION; else
	if (key.equals("ERROR"))         return ERROR; else
	if (key.equals("DOCURL"))        return DOCURL; else
	    throw new UnknownKeyException ("Unknown keyword " + key);
    }

    public static String Map (int key) throws UnknownKeyException {
	switch (key) {
	case HANDSHAKE:      return "HANDSHAKE";
	case MEETINGLIST:    return "MEETINGLIST";
	case GENERATE:       return "GENERATE";
	case TOOLS:          return "TOOLS";
	case CLOSE:          return "CLOSE";
	case CONFIRMATION:   return "CONFIRMATION";
	case ERROR:          return "ERROR";
	case DOCURL:         return "DOCURL";
	default: throw new UnknownKeyException ("Unknown key " + key);
	}

    }

    abstract public int GetNumberOfDataLinesFor (String key)
	throws UnknownKeyException;
}

// ****************************************************


