/*
  ServerCommunicationProtocol is a class which is a layer between
  raw data stream flow and structured messages.

  Alexander Podoplelov, RISC-Linz, 1998.
*/

// ****************************************************
import java.io.*;
import java.util.*;
import java.net.*;
// ****************************************************

public class ServerCommunicationProtocol extends CommunicationProtocol {
    
    public ServerCommunicationProtocol (BufferedReader in, BufferedWriter out){
	super (in, out);
    }

    // Getting the message according to protocol
    public Message GetMessage () 
	  throws SessionDoneException, UnknownKeyException, IOException {
	String key = in.readLine();
	if (key == null)
	    throw new SessionDoneException ("No keywords read.");
	Message message = new ServerMessage (key);
	int data_lines = message.GetNumberOfDataLinesFor (key);
	for (int i = 0; i < data_lines; i++) {
	    String data_line = in.readLine();
	    message.data.addElement (data_line);
	}
	Check (message);
	MeetingProtocolGenerator.Log ("Got \"" + key + "\".");
	return message;
    }

    public void Check (Message message) throws SessionDoneException {
	switch (message.key) {
	case message.GENERATE:
	    if (message.data.size() == 0)
		throw new SessionDoneException ("Message Integrity error");
	    try {
		Integer.parseInt ((String)message.data.elementAt (0));
		Integer.parseInt ((String)message.data.elementAt (1));
	    } catch (NumberFormatException e) {
		throw new SessionDoneException ("Message Integrity error");
	    }
	default:
	    return;
	}
    }
}

// ****************************************************

