/*
  ShelledConnection class provides a connection to a database with
  some service like sharing one connection between several consumers
  respecting synchronization ideas.

  Alexander Podoplelov, RISC-Linz, 1998.
*/

// ****************************************************
import java.io.*;
import java.net.*;
import java.lang.*;
import java.sql.*;
import java.util.*;
// ****************************************************

class ShelledConnection {
    private static  Object     reading_lock = new Object();

    // Returns the vector of ToolItems
    public Vector GetTools() throws SQLException {
	Vector      result = null;

	synchronized (reading_lock) {
	    Connection con  = MeetingSystemDB.getConnection ();
	    Statement  stmt = con.createStatement();
	    ResultSet  rset = stmt.executeQuery 
		("SELECT tool_id,tool_name FROM tool WHERE tool_id > 0");
	    while (rset.next()) {
		int    f_i = rset.getInt    ("tool_id");
		String f_s = rset.getString ("tool_name");
		if (result == null)
		    result = new Vector();
		result.addElement (new ToolItem (f_i, f_s));
	    }
	    stmt.close();
	    con.close();
	}
	return result;
    }
    // Returns the vector of ToolItems
    public Vector GetMeetings () throws SQLException {
	Vector result = null;

	synchronized (reading_lock) {
	    Connection con  = MeetingSystemDB.getConnection ();
	    Statement  stmt = con.createStatement();
	    ResultSet  rset = stmt.executeQuery
		("SELECT meeting_id,meeting_name,meeting_location," +
		 "meeting_start,meeting_end,meeting_description " +
		 "FROM meeting WHERE meeting_id > 1");
	    while (rset.next()) {
		int    f_id           = rset.getInt    ("meeting_id");
		String f_name         = rset.getString ("meeting_name");
		String f_location     = rset.getString ("meeting_location");
		java.sql.Date f_start = rset.getDate   ("meeting_start");
		java.sql.Date f_end   = rset.getDate   ("meeting_end");
		String f_description  = rset.getString ("meeting_description");
		if (result == null)
		    result = new Vector();
		result.addElement (new MeetingItem (f_id , f_name,
						    f_location, f_start,
						    f_end, f_description));
	    }
	    stmt.close();
	    con.close();
	}
	return result;
    }

    public Vector GetContributions (int meeting_id) throws SQLException {
	Vector result = null;

	synchronized (reading_lock) {
	    Connection con  = MeetingSystemDB.getConnection ();
	    Statement  stmt = con.createStatement();
	    ResultSet  rset = stmt.executeQuery
		("SELECT contribution_id,user_name," +
		 "parent,contribution_name,time_stamp,start_time,annotation " +
		 "FROM contribution WHERE contribution_id > 1 AND " +
		 "meeting_id = " + meeting_id);
	    while (rset.next()) {
		int    f_id           = rset.getInt    ("contribution_id");
		String f_uname        = rset.getString ("user_name");
		int    f_parent       = rset.getInt    ("parent");
		String f_name         = rset.getString ("contribution_name");
		Timestamp f_tstamp    = rset.getTimestamp ("time_stamp");
		String start_time = rset.getString   ("start_time");
		String  f_an          = rset.getString ("annotation");
		if (result == null)
		    result = new Vector();
		result.addElement
		    (new ContributionItem (f_id, meeting_id, f_uname,
					   f_parent, f_name, f_tstamp,
					   start_time,
					   f_an));
	    }
	    stmt.close();
	    con.close();
	}
	return result;
    }

    public Vector GetContributionParts (int contribution) throws SQLException {
	Vector result = null;

	synchronized (reading_lock) {
	    Connection con  = MeetingSystemDB.getConnection ();
	    Statement  stmt = con.createStatement();
	    ResultSet  rset = stmt.executeQuery
		("SELECT contribution_part_id,contribution_id,content_type," +
		 "content_subtype, storage_path " +
		 "FROM contribution_part WHERE contribution_id = " +
		 contribution);
// 		("SELECT contribution_part_id,contribution_id,content_type," +
// 		 "content_subtype, storage_path,time_stamp, file_name," +
// 		 "file_ext FROM contribution_part WHERE contribution_id = " +
// 		 contribution);
	    while (rset.next()) {
		int    id     = rset.getInt    ("contribution_part_id");
		int    c_id   = rset.getInt    ("contribution_id");
		String f_t    = rset.getString ("content_type");
		String f_st   = rset.getString ("content_subtype");
		String f_sp   = rset.getString ("storage_path");
// 		java.util.Date   f_ts   = rset.getDate   ("time_stamp");
// 		String f_n    = rset.getString ("file_name");
// 		String f_e    = rset.getString ("file_ext");
		if (result == null)
		    result = new Vector();
		result.addElement
		    (new ContributionPartItem (id, c_id, f_t, f_st, f_sp
// 					       ,
// 					       f_ts, f_n, f_e
					       ));
	    }
	    stmt.close();
	    con.close();
	}
	return result;
    }

    public Vector GetReferences (int contribution) throws SQLException {
	Vector result = null;

	synchronized (reading_lock) {
	    Connection con  = MeetingSystemDB.getConnection ();
	    Statement  stmt = con.createStatement();
	    ResultSet  rset = stmt.executeQuery
		("SELECT refers_to, is_referred_by " +
		 "FROM reference WHERE is_referred_by = " +
		 contribution);
	    while (rset.next()) {
		int    to     = rset.getInt    ("refers_to");
		if (result == null)
		    result = new Vector();
		result.addElement (new Reference (to));
	    }
	    stmt.close();
	    con.close();
	}
	return result;
    }
}

// ****************************************************
