/*
  ClientCommunicationProtocol is a class which is a layer between
  raw data stream flow and structured messages.

  Alexander Podoplelov, RISC-Linz, 1998.
*/

// ****************************************************
import java.io.*;
import java.util.*;
import java.net.*;
// ****************************************************

public class ClientCommunicationProtocol {
    private DataInputStream in;
    private PrintStream     out;
    
    public ClientCommunicationProtocol (DataInputStream in, PrintStream out) {
	this.in  = in;
	this.out = out;
    }

    // Getting the message according to protocol
    public Message GetMessage ()
	throws SessionDoneException, UnknownKeyException, IOException
    {
	String key = in.readLine();
	if (key == null)
	    throw new SessionDoneException ("No keywords read.");
	Message message = new ClientMessage (key);
	int data_lines = message.GetNumberOfDataLinesFor (key);
	if (data_lines != 0) {
	    if (data_lines < 0) { // data_lines to be read from the stream
		try {
		    key = in.readLine();
		    data_lines = Integer.parseInt (key);
		} catch (NullPointerException e1) {
		    throw new SessionDoneException
			("GetMessage: null data #-string");
		} catch (NumberFormatException e2) {
		    throw new SessionDoneException
			("GetMessage: wrong #-string: \"" + key + "\"");
		}
	    }
	    for (int i = 0; i < data_lines; i++) {
		String data_line = in.readLine();
		message.data.addElement (data_line);
	    }
	}
	return message;
    }

    public void SendMessage (int key) {
	try {
	    String skey  = Message.Map (key);
	    out.println (skey);
	} 
	catch (UnknownKeyException e) {}
    }

    public void SendMessage (int key, int data1, int data2) {
	try {
	    String skey  = Message.Map (key);
	    String sdata1  = Integer.toString (data1);
	    String sdata2  = Integer.toString (data2);
	    out.println (skey);
	    out.println (sdata1);
	    out.println (sdata2);
	} 
	catch (UnknownKeyException e) {}
    }
}

// ****************************************************
