/*
  MeetingProtocolNavigator is a class which provides GUI to the
  end user. It navigates the requests to the MeetingProtocol Server
  resulting in down-loading the generated documents into Web Browser
  frames the for further manipulating.

  Alexander Podoplelov, RISC-Linz, 1998.
*/

// ****************************************************
import java.applet.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.net.*;
// ****************************************************

public class MeetingProtocolNavigator extends Applet implements Runnable
{
    private static final int    FIRSTPANEL      = 1;
    private static final int    LASTPANEL       = 2;
    private static final String NEXT_LABEL      = "Next >>";
    private static final String DOWNLOAD_LABEL  = "Get";
    private static final String PRINTED_VERSION   = "Printable version (single file)";
    private static final String BROWSABLE_VERSION = "Browsable version (multiple files)";
    private static final String OPEN_SESSION      = "Open Session";
    private static final String CLOSE_SESSION     = "Close Session";

    public  static Insets          linsets          = null;
    //    public  static Image           image            = null;

    private static Color           FRAMECOLOR       = Color.gray;
    private static TextArea        Status           = null;
    private static Panel           Switcher         = null;
    private static Panel           ServicePanel     = null;
    private static Panel           ButtonPanel      = null;
    private static Button          Next             = null;
    private static Button          Prev             = null;
    private static Button          Disconnect       = null;
    private static Button          Help             = null;
    private static Button          ClearLog         = null;
    private static CheckboxGroup   cgroup           = null;

    private static int             cpanel           = 0;
    private static ClientCommunicationProtocol protocol = null;
    private static String          ServerHost       = null;
    private static String          IndexFrameName   = null;
    private static String          DataFrameName    = null;
    private static String          IndexFrameFile   = null;
    private static String          DataFrameFile    = null;
    private static String          DataBanner       = null;
    private static String          HelpFile         = null;
    private static int             port             = 0;
    private static int             nchars           = 30;

    private static Choice          Meetings         = null;
    private static AppletContext   appletcontext    = null;
    private static Socket          socket           = null;

    static {
	linsets = new Insets (3, 3, 3, 3);
    }

    public void init() {
	Panel p, p1;

	super.init();
	setBackground (Color.white);
	setForeground (Color.darkGray);
	
	//	if (image == null)
	//	    image = this.getImage(this.getDocumentBase(), "stein.gif");

	// ---------------------------------------------
	// Reading applet parameters
	// ---------------------------------------------
	ServerHost     = this.getParameter ("SERVER");
	IndexFrameName = this.getParameter ("INDEXFRAME");
	DataFrameName  = this.getParameter ("DATAFRAME");
	IndexFrameFile = this.getParameter ("INDEXFRAMEFILE");
	DataFrameFile  = this.getParameter ("DATAFRAMEFILE");
	DataBanner     = this.getParameter ("DATABANNER");
	HelpFile       = this.getParameter ("HELPFILE");

	int cport;
	try {
	    cport = Integer.parseInt(this.getParameter ("PORT"));
	} catch (NumberFormatException e) {
	    cport = 0;
	}
	if (cport != 0)
	    port = cport;
	else
	    port = MeetingProtocolGenerator.PORT;
	this.setLayout (new BorderLayout (3,3));

	// ---------------------------------------------
	// Creating GUI objects
	// ---------------------------------------------
	// Status Panel
	p = new FPanel (FRAMECOLOR);
	p.setLayout (new BorderLayout (3,3));
	p.add ("North", new Label ("Log"));
	Status = new TextArea (4, 40);
	Status.setEditable (false);
	p.add ("Center", Status);
	p1 = new FPanel (FRAMECOLOR);
	p1.setLayout (new FlowLayout (FlowLayout.LEFT, 3, 3));
	Disconnect = new Button (OPEN_SESSION);
	ClearLog = new Button ("Clear Log");
	Help = new Button ("Help");
	p1.add (Disconnect);
	p1.add (ClearLog);
	p1.add (Help);
	p.add ("South", p1);
	this.add ("Center", p);

	// Service Panel
	ServicePanel = new FPanel (FRAMECOLOR);
	ServicePanel.setLayout (new BorderLayout (3,3));

	ButtonPanel = new FPanel (FRAMECOLOR);
	ButtonPanel.setLayout (new FlowLayout (FlowLayout.CENTER, 3, 3));
	Prev = new Button ("<< Back");
	Next = new Button (NEXT_LABEL);
	Prev.disable();
	ButtonPanel.add (Prev);
	ButtonPanel.add (Next);
	ServicePanel.add ("South", ButtonPanel);
	ButtonPanel.hide();
	cpanel = FIRSTPANEL;

	Switcher = new FPanel (FRAMECOLOR);
	Switcher.setLayout (new CardLayout());
	ServicePanel.add ("Center", Switcher);

	// Logo Panel
	Image risc_logo = getImage (getDocumentBase(), "risc-logo.gif");
	p = new FPanel (FRAMECOLOR);
	p.setLayout (new GridBagLayout ());
	Arrange (p, new ImageComponent (risc_logo), 0, 0, 1, 2, 0.0, 0.0);
	Arrange (p, new Label("(c) RISC-Linz, 1998", Label.CENTER),
		 1, 0, 1, 1, 1.0, 1.0);
	Arrange (p, new Button("About"), 1, 1, 1, 1, 0.0, 0.0);
	Switcher.add ("Logo", p);

	p = new FPanel (FRAMECOLOR);
	p.setLayout (new GridBagLayout ());
	//	p.add (new ImageComponent (risc_logo));
	Arrange (p, new Label("Meeting Protocol Generator", Label.CENTER),
		  0, 0, 2, 1, 1.0, 1.0);
	Arrange (p, new Label("(c) RISC-Linz, 1998", Label.CENTER),
		 0, 1, 2, 1, 1.0, 1.0);
	Arrange (p, new Label("", Label.CENTER), 0, 2, 2, 1, 1.0, 1.0);
	Arrange (p, new Label("Authors: W. Schreiner, A. Podoplelov, F. Juen",
			 Label.CENTER), 0, 3, 2, 1, 1.0, 1.0);
	//	p.add (new Label("", Label.CENTER));
	Arrange (p, new Label ("  Research Institute for", Label.LEFT),
		 0, 4, 2, 1, 1.0, 1.0);
	Arrange (p, new Label ("  Symbolic Computation (RISC-Linz)",
			       Label.LEFT), 0, 5, 2, 1, 1.0, 1.0);
	Arrange (p, new Label ("  Johannes Kepler University, Linz, Austria",
			  Label.LEFT), 0, 6, 2, 1, 1.0, 1.0);
	Arrange (p, new Label("", Label.CENTER), 0, 7, 2, 1, 1.0, 1.0);
	Arrange (p, new Label("http://www.risc.uni-linz.ac.at", Label.LEFT),
		 0, 8, 2, 1, 1.0, 1.0);
	//	p.add (new Label("", Label.CENTER));
	Arrange (p, new Label("All rights for copying and distribution",
			 Label.LEFT), 0, 9, 1, 1, 0.0, 1.0);
	Arrange (p, new Label("are reserved by above owner.", Label.LEFT),
		 0, 10, 1, 1, 0.0, 1.0);
	Arrange (p, new Button("Ok"), 1, 9, 1, 2, 1.0, 1.0);
	Switcher.add ("About", p);
     

	// Wait Panel
	p = new FPanel (FRAMECOLOR);
	p.setLayout (new GridLayout (0,1));
	p.add (new Label(""));
	p.add (new Label("Wait for the document", Label.CENTER));
	p.add (new Label("to come up...", Label.CENTER));
	p.add (new Label(""));
	Switcher.add ("Wait", p);

	// First Panel
	p = new FPanel (FRAMECOLOR);
	p.setLayout (new GridBagLayout ());
	Arrange (p, new Label("Select a Meeting"), 0, 0, 1, 1, 1.0, 0.0);
	Meetings = new Choice();
	Arrange (p, Meetings, 0, 1, 1, 1, 0.0, 0.0);
	Switcher.add ("Meeting selection", p);


	// Second Panel
	p = new FPanel (FRAMECOLOR);
	p.setLayout (new GridBagLayout ());
	Arrange (p, new Label("Select document type"), 0, 0, 1, 1, 0.0, 0.0);
	cgroup = new CheckboxGroup();
	Arrange (p, new Checkbox(BROWSABLE_VERSION, cgroup, true),
		 0, 1, 1, 1, 1.0, 0.0);
	Arrange (p, new Checkbox(PRINTED_VERSION, cgroup, false),
		 0, 2, 1, 1, 1.0, 0.0);
	Switcher.add ("Option selection", p);

	((CardLayout)Switcher.getLayout()).show(Switcher, "Logo");
	this.add ("West", ServicePanel);

	appletcontext = getAppletContext();
	// end of initialization
    }

    public void Log (String s) {
	synchronized (Status) {
	    Status.appendText (s + "\n");
	}
    }

    public  Insets insets () {
	return linsets;
    }

    private static Thread MprotocolClient = null;

    public void NewSession() {
	if (MprotocolClient == null) {
	    MprotocolClient = new Thread(this);
	    MprotocolClient.start();
	}
    }

    public void start () {
        repaint();
    }

    public void stop () {
	if (MprotocolClient != null && MprotocolClient.isAlive())
	    MprotocolClient.stop();
	MprotocolClient = null;
    }

    public void resize (Dimension d) {
	System.out.println ("resize (Dimension d)");
	resize ( d.width, d.height);
    }

    public void resize (int width, int height) {
	super.resize (width, height);
	System.out.println ("resize (int " + width + ", int " + height + ")");
	layout();
	validate();
	//	paintAll (this.getGraphics());
    }

    public void paint (Graphics g) {
	System.out.println ("paint");
	update (g);
    }

    public void update (Graphics g) {
	Dimension d = size();
	System.out.println ("update : (" + d.width + ", " + d.height + ")");
	g.drawRect (1, 1, d.width - 2, d.height - 2);
    }

    // ------------------------------------------------------
    // The main loop
    // ------------------------------------------------------
    public void run () {

	try {
	    Log ("Connecting to the server " + ServerHost + ":" + port);
	    Connect();
	} catch (IOException e) {
	    Log ("Server is not running.");
	    Disconnect.setLabel (OPEN_SESSION);
	    Disconnect.enable();
	    ButtonPanel.hide();
	    ServicePanel.validate();
	    MprotocolClient = null;
	    return;
	} catch (SecurityException e1) {
	    Log ("SecurityManager reports: " + e1.toString());
	    return;
	}
	Log ("Ok.\nDownloading data.");
	try {
	    DownloadData();
	} catch (UnknownKeyException e1) {
	    Log	("UnknownKeyException:" + e1.toString() + ".");
	    return;
	} catch (SessionDoneException e2) {
	    Log	("SessionDoneException: " + e2.toString() + ".");
	    Disconnect();
	    DisconnectState();
	    return;
	}

	// Now we are able to allow some options
	cpanel = FIRSTPANEL;
	((CardLayout)Switcher.getLayout()).show(Switcher, "Meeting selection");
	Next.setLabel (NEXT_LABEL);
	Disconnect.enable();
	ButtonPanel.show();

	ServicePanel.validate();

// 	while (socket != null) {
// 	    try {
// 		Thread.sleep (1000);
// 	    } catch (InterruptedException e) {}
// 	    //	    System.out.print ("\u0007");
// 	}
    }

    private void WaitForDoc() {
	ButtonPanel.hide();
	Disconnect.disable();
	((CardLayout)Switcher.getLayout()).show(Switcher, "Wait");
	ServicePanel.validate();
	try { Thread.sleep (500); } catch (InterruptedException e) {}
    }

    private void WaitForDocDone() {
	ButtonPanel.show();
	Disconnect.enable();
	((CardLayout)Switcher.getLayout()).show(Switcher, "Option selection");
	ServicePanel.validate();
    }

    private void DefaultFrameIndex() {
	URL url = null;
	try {
	    url = new URL (getCodeBase().toString() + IndexFrameFile);
	} catch (MalformedURLException e) {
	    Log (e.toString());
	}
	appletcontext.showDocument (url, IndexFrameName);
    }
    private void DefaultFrameData() {
	URL url = null;
	try {
	    url = new URL (getCodeBase().toString() + DataFrameFile);
	} catch (MalformedURLException e) {
	    Log (e.toString());
	}
	appletcontext.showDocument (url, DataFrameName);
    }

    private void ShowHelp() {
	URL url = null;
	try {
	    url = new URL (getCodeBase().toString() + HelpFile);
	} catch (MalformedURLException e) {
	    Log (e.toString());
	}
	appletcontext.showDocument (url, "_blank");
    }

    private void Connect() throws IOException, SecurityException {
	System.getSecurityManager().checkConnect(ServerHost, port);
	socket = new Socket (ServerHost, port);
	protocol = new ClientCommunicationProtocol
	    (new DataInputStream(socket.getInputStream()),
	     new PrintStream(socket.getOutputStream())	    
	     );
    }

    private void Disconnect() {
	try { socket.close(); } catch (IOException e) { Log (e.toString()); }
	socket   = null;
	protocol = null;
	Container c = Meetings.getParent();
	c.remove (Meetings);
	Meetings = new Choice();
	Arrange (c, Meetings, 0, 1, 1, 1, 0.0, 0.0);
    }

    private void Download (Choice choice, int query) 
	throws UnknownKeyException, SessionDoneException
    {
	Message     message   = null;
	FontMetrics fm        = getGraphics().getFontMetrics();
	Insets      c_insets  = ((FPanel)Switcher.getComponent(0)).insets();
	int         width     = Switcher.getComponent(0).size().width -
	    choice.minimumSize().width - c_insets.left - c_insets.right;

	try {
	    protocol.SendMessage (query);
	    message = protocol.GetMessage(); // First message
	    if (message.key == Message.CONFIRMATION &&
		message.data.elementAt(0).equals(Message.Map(query)))
		{
		    Log ("Got confirmation for " + Message.Map(query) +
			 " command.");
		    message = protocol.GetMessage();// Second message
		    if (message.data.size() > 0) {
			for (int i = 0; i < message.data.size(); i++) {
			    int    j, w;
			    String item = (String)message.data.elementAt(i);
			    StringBuffer sb = new StringBuffer (item);
			    // Shrinking the string according to the "choice"
			    // length 
			    for (w = 0,j = 0;
				 w < width && j < item.length();
				 j++)
				w += fm.charWidth (sb.charAt (j));
			    nchars = j;
			    if (item.length() > nchars)
				choice.addItem (item.substring(0, nchars));
			    else
				choice.addItem (item);
			}
		    }
		    Log	("Got " + message.data.size() + " items.");
		}
	} catch (IOException e) {
	    throw new SessionDoneException ("Server closed the connection");
	}
    }

    private void DownloadData() 
	throws UnknownKeyException, SessionDoneException
    {
	Download (Meetings, Message.MEETINGLIST);
    }

    private void DisconnectState () {
	Disconnect.setLabel (OPEN_SESSION);
	Disconnect.enable();
	ButtonPanel.hide();
	((CardLayout)Switcher.getLayout()).show (Switcher, "Logo");
	ServicePanel.validate();
    }

    // ------------------------------------------------------
    // Operate on the actions
    // ------------------------------------------------------
    public boolean action (Event event, Object what) {
	if (event.target instanceof Button) {
	    if (event.target == Help) {
		ShowHelp ();
		return true;
	    } else if (((Button)event.target).getLabel().equals("About")) {
		// "About" 
		((CardLayout)Switcher.getLayout()).show(Switcher, "About");
		return true;
	    } else if (((Button)event.target).getLabel().equals("Ok")) {// "Ok"
		((CardLayout)Switcher.getLayout()).show(Switcher, "Logo");
		return true;
	    } else if (event.target == Prev) { // "Prev" button is pressed
		((CardLayout)Switcher.getLayout()).previous(Switcher);
		if (cpanel == LASTPANEL)
		    Next.setLabel (NEXT_LABEL);
		if (cpanel == FIRSTPANEL + 1)
		    Prev.disable();
		cpanel--;
		return true;
	    } else if (event.target == Next) { // "Next" button is pressed
		if (cpanel == LASTPANEL) {
		    // Now it's time download the document
		    GetTheDocument (cgroup.getCurrent().getLabel());
		    // DefaultFrameData();
		} else {
		    ((CardLayout)Switcher.getLayout()).next(Switcher);
		    if (cpanel == FIRSTPANEL)
			Prev.enable();
		    if (cpanel == LASTPANEL - 1)
			Next.setLabel (DOWNLOAD_LABEL);
		    cpanel++;
		}
		return true;
	    } else if (event.target == Disconnect) {
		//"Discon."button is pressed
		Disconnect.disable();
		if (Disconnect.getLabel().equals(CLOSE_SESSION)) {
		    Log ("Shutting down.");
		    protocol.SendMessage (Message.CLOSE);
		    try {
			Message m = protocol.GetMessage();
			if (m.key == Message.CONFIRMATION &&
			    m.data.elementAt (0).equals
			    (Message.Map(Message.CLOSE)))
			    Log ("Got confirmation for CLOSE");
		    }
		    catch (IOException e1) { Log (e1.toString()); }
		    catch (UnknownKeyException e2) { Log (e2.toString()); }
		    catch (SessionDoneException e3) { Log (e3.toString()); }
		    Disconnect();
		    stop();
		    DisconnectState();
		} else if (Disconnect.getLabel().equals(OPEN_SESSION)) {
		    Disconnect.setLabel (CLOSE_SESSION);
		    NewSession();
		    Prev.disable();
		    DefaultFrameIndex();
		    DefaultFrameData();
		}
		return true;
	    } else if (event.target == ClearLog) {
		// "ClearLog" button is pressed
		Log ("Clearing the log.");
		Status.replaceText("", 0, Status.getText().length());
		return true;
	    }
        } else if (event.target == Meetings) {   //Meeting is selected
	    Log	("Selected \"" + Meetings.getSelectedItem() + "\"");
	    return true;
	} else if (event.target instanceof Checkbox) {// Checkboxes are clicked
	    Log	("Selected \"" + cgroup.getCurrent().getLabel() + "\"");
	    return true;
	}
	return false;
    }

    static void Arrange(Container cont, Component comp,
            int x, int y, int w, int h, double weightx, double weighty) {
 
	GridBagLayout gbl = (GridBagLayout)cont.getLayout();
        GridBagConstraints c = new GridBagConstraints();
 
        c.fill          = GridBagConstraints.BOTH;
        c.gridx         = x;
        c.gridy         = y;
        c.gridwidth     = w;
        c.gridheight    = h;
        c.weightx       = weightx;
        c.weighty       = weighty;
        cont.add (comp);
        gbl.setConstraints (comp, c);
    }

    public void GetTheDocument (String lab) {
	int ver;
	boolean error = false;

	WaitForDoc();
	Log ("Requesting the document.");
	if (lab.equals (PRINTED_VERSION))
	    ver = Generator.PRINTED_VERSION;
	else if (lab.equals (BROWSABLE_VERSION))
	    ver = Generator.BROWSABLE_VERSION;
	else
	    ver = 0;

	try {
	    protocol.SendMessage
		(Message.GENERATE, Meetings.getSelectedIndex(), ver);
	    Message message = protocol.GetMessage(); // First message
	    if (message != null && message.key == Message.CONFIRMATION &&
		message.data.elementAt(0).equals
		(Message.Map(Message.GENERATE)))
		{
		    Log ("Got confirmation for GENERATE command.");
		    message = protocol.GetMessage();// Second message
		    if (message.data.size() > 0) {
			URL url = null;
			URL banner = null;
			try {
			    url = new URL ((String)message.data.elementAt (0));
			    File f = new File (url.getFile());
			    File fr = new File (f.getParent(), DataBanner);
			    banner = new URL (url.getProtocol(), url.getHost(),
					      fr.getPath());
			    Log ("Loading " + url.toString());
			} catch (MalformedURLException e) {
			    Log (e.toString());
			}
			appletcontext.showDocument (url, IndexFrameName);
			appletcontext.showDocument (banner, DataFrameName);
		    }
		}
	} catch (IOException e1) {
	    error = true;
	    Log (e1.toString());
	} catch (UnknownKeyException e2) {
	    error = true;
	    Log (e2.toString());
	} catch (SessionDoneException e3) {
	    error = true;
	    Log (e3.toString());
	}
	if (error) {
	    Disconnect();
	    stop();
	    DisconnectState();
	} else
	    WaitForDocDone();
    }
}

// ----------------------------------------------------
class FPanel extends Panel {
    private Color color;

    public FPanel (Color color) {
	super();
	this.color = color;
    }

    public  Insets insets () {
	return MeetingProtocolNavigator.linsets;
    }

    public void paint(Graphics g) {
	Dimension d = size();
	g.setColor (color);
	g.drawRect (1, 1, d.width - 2, d.height - 2);
	/*
	int i, j, w, h;
	w = MeetingProtocolNavigator.image.getWidth(this);
	h = MeetingProtocolNavigator.image.getHeight(this);
	if (w > 0 && h > 0)
	    for (i = 0; i < d.width; i += w)
		for (j = 0; j < d.height; j += h)
		    g.drawImage
			(MeetingProtocolNavigator.image, i+2, j+2, this);
	*/
    }
}

// ---------------------------------------------------

class ImageComponent extends Canvas {
    Image     image;
 
    public ImageComponent (Image image) {
	if (image != null) {
	    this.image = image;
	    waitForImage();
	}
    }
 
    private void waitForImage () {
        try {
            // Use a media tracker object to wait until all the pixels 
            // have been retrieved.
            MediaTracker tracker = new MediaTracker(this);
            tracker.addImage(image, 0);
            tracker.waitForID(0);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Dimension preferredSize() {
	if (image == null) return new Dimension (0, 0);
	Insets i = insets();
	return new Dimension
	    (image.getWidth (this) + i.left + i.right,
	     image.getHeight (this) + i.top + i.bottom);
    }

    public Dimension minimumSize() {
	if (image == null) return new Dimension (0, 0);
	Insets i = insets();
	return new Dimension
	    (image.getWidth (this) + i.left + i.right,
	     image.getHeight (this) + i.top + i.bottom);
    }

    public  Insets insets () {
	return MeetingProtocolNavigator.linsets;
    }

    public void paint(Graphics g) {
	if (image == null) return;
	Dimension d = size();
	Insets i = insets();
	int w, h;
	w = image.getWidth (this);
	h = image.getHeight (this);
        g.drawImage(image, (d.width - i.left - i.right - w)/2,
		    (d.height - i.top - i.bottom - h)/2, this);
    }
}

// ****************************************************
